@@ -12,53 +12,60 @@
 # language governing permissions and limitations under the License.
 import copy
 import logging
+import re
 import threading
 
-from botocore.compat import six
-
-from s3transfer.constants import KB, MB
-from s3transfer.constants import ALLOWED_DOWNLOAD_ARGS
-from s3transfer.utils import get_callbacks
-from s3transfer.utils import signal_transferring
-from s3transfer.utils import signal_not_transferring
-from s3transfer.utils import CallArgs
-from s3transfer.utils import OSUtils
-from s3transfer.utils import TaskSemaphore
-from s3transfer.utils import SlidingWindowSemaphore
-from s3transfer.exceptions import CancelledError
-from s3transfer.exceptions import FatalError
-from s3transfer.futures import IN_MEMORY_DOWNLOAD_TAG
-from s3transfer.futures import IN_MEMORY_UPLOAD_TAG
-from s3transfer.futures import BoundedExecutor
-from s3transfer.futures import TransferFuture
-from s3transfer.futures import TransferMeta
-from s3transfer.futures import TransferCoordinator
-from s3transfer.download import DownloadSubmissionTask
-from s3transfer.upload import UploadSubmissionTask
+from s3transfer.bandwidth import BandwidthLimiter, LeakyBucket
+from s3transfer.constants import (
+    ALLOWED_DOWNLOAD_ARGS,
+    FULL_OBJECT_CHECKSUM_ARGS,
+    KB,
+    MB,
+)
 from s3transfer.copies import CopySubmissionTask
 from s3transfer.delete import DeleteSubmissionTask
-from s3transfer.bandwidth import LeakyBucket
-from s3transfer.bandwidth import BandwidthLimiter
-
+from s3transfer.download import DownloadSubmissionTask
+from s3transfer.exceptions import CancelledError, FatalError
+from s3transfer.futures import (
+    IN_MEMORY_DOWNLOAD_TAG,
+    IN_MEMORY_UPLOAD_TAG,
+    BoundedExecutor,
+    TransferCoordinator,
+    TransferFuture,
+    TransferMeta,
+)
+from s3transfer.upload import UploadSubmissionTask
+from s3transfer.utils import (
+    CallArgs,
+    OSUtils,
+    SlidingWindowSemaphore,
+    TaskSemaphore,
+    get_callbacks,
+    set_default_checksum_algorithm,
+    signal_not_transferring,
+    signal_transferring,
+)
 
 logger = logging.getLogger(__name__)
 
 
-class TransferConfig(object):
-    def __init__(self,
-                 multipart_threshold=8 * MB,
-                 multipart_chunksize=8 * MB,
-                 max_request_concurrency=10,
-                 max_submission_concurrency=5,
-                 max_request_queue_size=1000,
-                 max_submission_queue_size=1000,
-                 max_io_queue_size=1000,
-                 io_chunksize=256 * KB,
-                 num_download_attempts=5,
-                 max_in_memory_upload_chunks=10,
-                 max_in_memory_download_chunks=10,
-                 max_bandwidth=None):
-        """Configurations for the transfer mangager
+class TransferConfig:
+    def __init__(
+        self,
+        multipart_threshold=8 * MB,
+        multipart_chunksize=8 * MB,
+        max_request_concurrency=10,
+        max_submission_concurrency=5,
+        max_request_queue_size=1000,
+        max_submission_queue_size=1000,
+        max_io_queue_size=1000,
+        io_chunksize=256 * KB,
+        num_download_attempts=5,
+        max_in_memory_upload_chunks=10,
+        max_in_memory_download_chunks=10,
+        max_bandwidth=None,
+    ):
+        """Configurations for the transfer manager
 
         :param multipart_threshold: The threshold for which multipart
             transfers occur.
@@ -70,7 +77,7 @@ class TransferConfig(object):
             processing a call to a TransferManager method. Processing a
             call usually entails determining which S3 API requests that need
             to be enqueued, but does **not** entail making any of the
-            S3 API data transfering requests needed to perform the transfer.
+            S3 API data transferring requests needed to perform the transfer.
             The threads controlled by ``max_request_concurrency`` is
             responsible for that.
 
@@ -78,17 +85,14 @@ class TransferConfig(object):
             becomes a multipart transfer.
 
         :param max_request_queue_size: The maximum amount of S3 API requests
-            that can be queued at a time. A value of zero means that there
-            is no maximum.
+            that can be queued at a time.
 
         :param max_submission_queue_size: The maximum amount of
-            TransferManager method calls that can be queued at a time. A value
-            of zero means that there is no maximum.
+            TransferManager method calls that can be queued at a time.
 
         :param max_io_queue_size: The maximum amount of read parts that
-            can be queued to be written to disk per download. A value of zero
-            means that there is no maximum. The default size for each element
-            in this queue is 8 KB.
+            can be queued to be written to disk per download. The default
+            size for each elementin this queue is 8 KB.
 
         :param io_chunksize: The max size of each chunk in the io queue.
             Currently, this is size used when reading from the downloaded
@@ -96,9 +100,9 @@ class TransferConfig(object):
 
         :param num_download_attempts: The number of download attempts that
             will be tried upon errors with downloading an object in S3. Note
-            that these retries account for errors that occur when streamming
+            that these retries account for errors that occur when streaming
             down the data from s3 (i.e. socket errors and read timeouts that
-            occur after recieving an OK response from s3).
+            occur after receiving an OK response from s3).
             Other retryable exceptions such as throttling errors and 5xx errors
             are already retried by botocore (this default is 5). The
             ``num_download_attempts`` does not take into account the
@@ -122,7 +126,7 @@ class TransferConfig(object):
 
         :param max_in_memory_download_chunks: The number of chunks that can
             be buffered in memory and **not** in the io queue at a time for all
-            ongoing dowload requests. This pertains specifically to file-like
+            ongoing download requests. This pertains specifically to file-like
             objects that cannot be seeked. The total maximum memory footprint
             due to a in-memory download chunks is roughly equal to:
 
@@ -147,29 +151,35 @@ class TransferConfig(object):
         self._validate_attrs_are_nonzero()
 
     def _validate_attrs_are_nonzero(self):
-        for attr, attr_val, in self.__dict__.items():
+        for attr, attr_val in self.__dict__.items():
             if attr_val is not None and attr_val <= 0:
                 raise ValueError(
-                    'Provided parameter %s of value %s must be greater than '
-                    '0.' % (attr, attr_val))
+                    f'Provided parameter {attr} of value {attr_val} must '
+                    'be greater than 0.'
+                )
 
 
-class TransferManager(object):
+class TransferManager:
     ALLOWED_DOWNLOAD_ARGS = ALLOWED_DOWNLOAD_ARGS
 
-    ALLOWED_UPLOAD_ARGS = [
+    _ALLOWED_SHARED_ARGS = [
         'ACL',
         'CacheControl',
+        'ChecksumAlgorithm',
         'ContentDisposition',
         'ContentEncoding',
         'ContentLanguage',
         'ContentType',
+        'ExpectedBucketOwner',
         'Expires',
         'GrantFullControl',
         'GrantRead',
         'GrantReadACP',
         'GrantWriteACP',
         'Metadata',
+        'ObjectLockLegalHoldStatus',
+        'ObjectLockMode',
+        'ObjectLockRetainUntilDate',
         'RequestPayer',
         'ServerSideEncryption',
         'StorageClass',
@@ -179,10 +189,19 @@ class TransferManager(object):
         'SSEKMSKeyId',
         'SSEKMSEncryptionContext',
         'Tagging',
-        'WebsiteRedirectLocation'
+        'WebsiteRedirectLocation',
     ]
 
-    ALLOWED_COPY_ARGS = ALLOWED_UPLOAD_ARGS + [
+    ALLOWED_UPLOAD_ARGS = (
+        _ALLOWED_SHARED_ARGS
+        + [
+            'ChecksumType',
+            'MpuObjectSize',
+        ]
+        + FULL_OBJECT_CHECKSUM_ARGS
+    )
+
+    ALLOWED_COPY_ARGS = _ALLOWED_SHARED_ARGS + [
         'CopySourceIfMatch',
         'CopySourceIfModifiedSince',
         'CopySourceIfNoneMatch',
@@ -198,8 +217,18 @@ class TransferManager(object):
         'MFA',
         'VersionId',
         'RequestPayer',
+        'ExpectedBucketOwner',
     ]
 
+    VALIDATE_SUPPORTED_BUCKET_VALUES = True
+
+    _UNSUPPORTED_BUCKET_PATTERNS = {
+        'S3 Object Lambda': re.compile(
+            r'^arn:(aws).*:s3-object-lambda:[a-z\-0-9]+:[0-9]{12}:'
+            r'accesspoint[/:][a-zA-Z0-9\-]{1,63}'
+        ),
+    }
+
     def __init__(self, client, config=None, osutil=None, executor_cls=None):
         """A transfer manager interface for Amazon S3
 
@@ -229,11 +258,13 @@ class TransferManager(object):
             max_num_threads=self._config.max_request_concurrency,
             tag_semaphores={
                 IN_MEMORY_UPLOAD_TAG: TaskSemaphore(
-                    self._config.max_in_memory_upload_chunks),
+                    self._config.max_in_memory_upload_chunks
+                ),
                 IN_MEMORY_DOWNLOAD_TAG: SlidingWindowSemaphore(
-                    self._config.max_in_memory_download_chunks)
+                    self._config.max_in_memory_download_chunks
+                ),
             },
-            executor_cls=executor_cls
+            executor_cls=executor_cls,
         )
 
         # The executor responsible for submitting the necessary tasks to
@@ -241,8 +272,7 @@ class TransferManager(object):
         self._submission_executor = BoundedExecutor(
             max_size=self._config.max_submission_queue_size,
             max_num_threads=self._config.max_submission_concurrency,
-            executor_cls=executor_cls
-
+            executor_cls=executor_cls,
         )
 
         # There is one thread available for writing to disk. It will handle
@@ -250,7 +280,7 @@ class TransferManager(object):
         self._io_executor = BoundedExecutor(
             max_size=self._config.max_io_queue_size,
             max_num_threads=1,
-            executor_cls=executor_cls
+            executor_cls=executor_cls,
         )
 
         # The component responsible for limiting bandwidth usage if it
@@ -258,7 +288,8 @@ class TransferManager(object):
         self._bandwidth_limiter = None
         if self._config.max_bandwidth is not None:
             logger.debug(
-                'Setting max_bandwidth to %s', self._config.max_bandwidth)
+                'Setting max_bandwidth to %s', self._config.max_bandwidth
+            )
             leaky_bucket = LeakyBucket(self._config.max_bandwidth)
             self._bandwidth_limiter = BandwidthLimiter(leaky_bucket)
 
@@ -298,23 +329,30 @@ class TransferManager(object):
         :rtype: s3transfer.futures.TransferFuture
         :returns: Transfer future representing the upload
         """
-        if extra_args is None:
-            extra_args = {}
+
+        extra_args = extra_args.copy() if extra_args else {}
         if subscribers is None:
             subscribers = []
         self._validate_all_known_args(extra_args, self.ALLOWED_UPLOAD_ARGS)
+        self._validate_if_bucket_supported(bucket)
+        self._add_operation_defaults(extra_args)
         call_args = CallArgs(
-            fileobj=fileobj, bucket=bucket, key=key, extra_args=extra_args,
-            subscribers=subscribers
+            fileobj=fileobj,
+            bucket=bucket,
+            key=key,
+            extra_args=extra_args,
+            subscribers=subscribers,
         )
         extra_main_kwargs = {}
         if self._bandwidth_limiter:
             extra_main_kwargs['bandwidth_limiter'] = self._bandwidth_limiter
         return self._submit_transfer(
-            call_args, UploadSubmissionTask, extra_main_kwargs)
+            call_args, UploadSubmissionTask, extra_main_kwargs
+        )
 
-    def download(self, bucket, key, fileobj, extra_args=None,
-                 subscribers=None):
+    def download(
+        self, bucket, key, fileobj, extra_args=None, subscribers=None
+    ):
         """Downloads a file from S3
 
         :type bucket: str
@@ -345,18 +383,30 @@ class TransferManager(object):
         if subscribers is None:
             subscribers = []
         self._validate_all_known_args(extra_args, self.ALLOWED_DOWNLOAD_ARGS)
+        self._validate_if_bucket_supported(bucket)
         call_args = CallArgs(
-            bucket=bucket, key=key, fileobj=fileobj, extra_args=extra_args,
-            subscribers=subscribers
+            bucket=bucket,
+            key=key,
+            fileobj=fileobj,
+            extra_args=extra_args,
+            subscribers=subscribers,
         )
         extra_main_kwargs = {'io_executor': self._io_executor}
         if self._bandwidth_limiter:
             extra_main_kwargs['bandwidth_limiter'] = self._bandwidth_limiter
         return self._submit_transfer(
-            call_args, DownloadSubmissionTask, extra_main_kwargs)
+            call_args, DownloadSubmissionTask, extra_main_kwargs
+        )
 
-    def copy(self, copy_source, bucket, key, extra_args=None,
-             subscribers=None, source_client=None):
+    def copy(
+        self,
+        copy_source,
+        bucket,
+        key,
+        extra_args=None,
+        subscribers=None,
+        source_client=None,
+    ):
         """Copies a file in S3
 
         :type copy_source: dict
@@ -398,10 +448,16 @@ class TransferManager(object):
         if source_client is None:
             source_client = self._client
         self._validate_all_known_args(extra_args, self.ALLOWED_COPY_ARGS)
+        if isinstance(copy_source, dict):
+            self._validate_if_bucket_supported(copy_source.get('Bucket'))
+        self._validate_if_bucket_supported(bucket)
         call_args = CallArgs(
-            copy_source=copy_source, bucket=bucket, key=key,
-            extra_args=extra_args, subscribers=subscribers,
-            source_client=source_client
+            copy_source=copy_source,
+            bucket=bucket,
+            key=key,
+            extra_args=extra_args,
+            subscribers=subscribers,
+            source_client=source_client,
         )
         return self._submit_transfer(call_args, CopySubmissionTask)
 
@@ -432,28 +488,54 @@ class TransferManager(object):
         if subscribers is None:
             subscribers = []
         self._validate_all_known_args(extra_args, self.ALLOWED_DELETE_ARGS)
+        self._validate_if_bucket_supported(bucket)
         call_args = CallArgs(
-            bucket=bucket, key=key, extra_args=extra_args,
-            subscribers=subscribers
+            bucket=bucket,
+            key=key,
+            extra_args=extra_args,
+            subscribers=subscribers,
         )
         return self._submit_transfer(call_args, DeleteSubmissionTask)
 
+    def _validate_if_bucket_supported(self, bucket):
+        # s3 high level operations don't support some resources
+        # (eg. S3 Object Lambda) only direct API calls are available
+        # for such resources
+        if self.VALIDATE_SUPPORTED_BUCKET_VALUES:
+            for resource, pattern in self._UNSUPPORTED_BUCKET_PATTERNS.items():
+                match = pattern.match(bucket)
+                if match:
+                    raise ValueError(
+                        f'TransferManager methods do not support {resource} '
+                        'resource. Use direct client calls instead.'
+                    )
+
     def _validate_all_known_args(self, actual, allowed):
         for kwarg in actual:
             if kwarg not in allowed:
                 raise ValueError(
-                    "Invalid extra_args key '%s', "
-                    "must be one of: %s" % (
-                        kwarg, ', '.join(allowed)))
+                    "Invalid extra_args key '{}', must be one of: {}".format(
+                        kwarg, ', '.join(allowed)
+                    )
+                )
 
-    def _submit_transfer(self, call_args, submission_task_cls,
-                         extra_main_kwargs=None):
+    def _add_operation_defaults(self, extra_args):
+        if (
+            self.client.meta.config.request_checksum_calculation
+            == "when_supported"
+        ):
+            set_default_checksum_algorithm(extra_args)
+
+    def _submit_transfer(
+        self, call_args, submission_task_cls, extra_main_kwargs=None
+    ):
         if not extra_main_kwargs:
             extra_main_kwargs = {}
 
         # Create a TransferFuture to return back to the user
         transfer_future, components = self._get_future_with_components(
-            call_args)
+            call_args
+        )
 
         # Add any provided done callbacks to the created transfer future
         # to be invoked on the transfer future being complete.
@@ -462,14 +544,15 @@ class TransferManager(object):
 
         # Get the main kwargs needed to instantiate the submission task
         main_kwargs = self._get_submission_task_main_kwargs(
-            transfer_future, extra_main_kwargs)
+            transfer_future, extra_main_kwargs
+        )
 
         # Submit a SubmissionTask that will submit all of the necessary
         # tasks needed to complete the S3 transfer.
         self._submission_executor.submit(
             submission_task_cls(
                 transfer_coordinator=components['coordinator'],
-                main_kwargs=main_kwargs
+                main_kwargs=main_kwargs,
             )
         )
 
@@ -484,27 +567,30 @@ class TransferManager(object):
         transfer_coordinator = TransferCoordinator(transfer_id=transfer_id)
         # Track the transfer coordinator for transfers to manage.
         self._coordinator_controller.add_transfer_coordinator(
-            transfer_coordinator)
+            transfer_coordinator
+        )
         # Also make sure that the transfer coordinator is removed once
         # the transfer completes so it does not stick around in memory.
         transfer_coordinator.add_done_callback(
             self._coordinator_controller.remove_transfer_coordinator,
-            transfer_coordinator)
+            transfer_coordinator,
+        )
         components = {
             'meta': TransferMeta(call_args, transfer_id=transfer_id),
-            'coordinator': transfer_coordinator
+            'coordinator': transfer_coordinator,
         }
         transfer_future = TransferFuture(**components)
         return transfer_future, components
 
     def _get_submission_task_main_kwargs(
-            self, transfer_future, extra_main_kwargs):
+        self, transfer_future, extra_main_kwargs
+    ):
         main_kwargs = {
             'client': self._client,
             'config': self._config,
             'osutil': self._osutil,
             'request_executor': self._request_executor,
-            'transfer_future': transfer_future
+            'transfer_future': transfer_future,
         }
         main_kwargs.update(extra_main_kwargs)
         return main_kwargs
@@ -513,11 +599,13 @@ class TransferManager(object):
         # Register handlers to enable/disable callbacks on uploads.
         event_name = 'request-created.s3'
         self._client.meta.events.register_first(
-            event_name, signal_not_transferring,
-            unique_id='s3upload-not-transferring')
+            event_name,
+            signal_not_transferring,
+            unique_id='s3upload-not-transferring',
+        )
         self._client.meta.events.register_last(
-            event_name, signal_transferring,
-            unique_id='s3upload-transferring')
+            event_name, signal_transferring, unique_id='s3upload-transferring'
+        )
 
     def __enter__(self):
         return self
@@ -530,7 +618,7 @@ class TransferManager(object):
         # all of the inprogress futures in the shutdown.
         if exc_type:
             cancel = True
-            cancel_msg = six.text_type(exc_value)
+            cancel_msg = str(exc_value)
             if not cancel_msg:
                 cancel_msg = repr(exc_value)
             # If it was a KeyboardInterrupt, the cancellation was initiated
@@ -581,7 +669,7 @@ class TransferManager(object):
             self._io_executor.shutdown()
 
 
-class TransferCoordinatorController(object):
+class TransferCoordinatorController:
     def __init__(self):
         """Abstraction to control all transfer coordinators
 
@@ -611,7 +699,7 @@ class TransferCoordinatorController(object):
             self._tracked_transfer_coordinators.add(transfer_coordinator)
 
     def remove_transfer_coordinator(self, transfer_coordinator):
-        """Remove a transfer coordinator from cancelation consideration
+        """Remove a transfer coordinator from cancellation consideration
 
         Typically, this method is invoked by the transfer coordinator itself
         to remove its self when it completes its transfer.
@@ -640,7 +728,7 @@ class TransferCoordinatorController(object):
     def wait(self):
         """Wait until there are no more inprogress transfers
 
-        This will not stop when failures are encountered and not propogate any
+        This will not stop when failures are encountered and not propagate any
         of these errors from failed transfers, but it can be interrupted with
         a KeyboardInterrupt.
         """
@@ -656,7 +744,8 @@ class TransferCoordinatorController(object):
             if transfer_coordinator:
                 logger.debug(
                     'On KeyboardInterrupt was waiting for %s',
-                    transfer_coordinator)
+                    transfer_coordinator,
+                )
             raise
         except Exception:
             # A general exception could have been thrown because
