@@ -11,32 +11,38 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import copy
-import os
-import tempfile
-import shutil
 import glob
+import os
+import shutil
+import tempfile
+import time
+from io import BytesIO
 
 from botocore.exceptions import ClientError
 
-from tests import StreamWithError
-from tests import FileSizeProvider
-from tests import RecordingSubscriber
-from tests import RecordingOSUtils
-from tests import NonSeekableWriter
-from tests import BaseGeneralInterfaceTest
-from tests import skip_if_windows
-from tests import skip_if_using_serial_implementation
-from s3transfer.compat import six
 from s3transfer.compat import SOCKET_ERROR
-from s3transfer.exceptions import RetriesExceededError
-from s3transfer.manager import TransferManager
-from s3transfer.manager import TransferConfig
-from s3transfer.download import GetObjectTask
+from s3transfer.exceptions import (
+    RetriesExceededError,
+    S3DownloadFailedError,
+    S3ValidationError,
+)
+from s3transfer.manager import TransferConfig, TransferManager
+from tests import (
+    BaseGeneralInterfaceTest,
+    ETagProvider,
+    FileSizeProvider,
+    NonSeekableWriter,
+    RecordingOSUtils,
+    RecordingSubscriber,
+    StreamWithError,
+    skip_if_using_serial_implementation,
+    skip_if_windows,
+)
 
 
 class BaseDownloadTest(BaseGeneralInterfaceTest):
     def setUp(self):
-        super(BaseDownloadTest, self).setUp()
+        super().setUp()
         self.config = TransferConfig(max_request_concurrency=1)
         self._manager = TransferManager(self.client, self.config)
 
@@ -47,15 +53,16 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         # Initialize some default arguments
         self.bucket = 'mybucket'
         self.key = 'mykey'
+        self.etag = 'myetag'
         self.extra_args = {}
         self.subscribers = []
 
         # Create a stream to read from
         self.content = b'my content'
-        self.stream = six.BytesIO(self.content)
+        self.stream = BytesIO(self.content)
 
     def tearDown(self):
-        super(BaseDownloadTest, self).tearDown()
+        super().tearDown()
         shutil.rmtree(self.tempdir)
 
     @property
@@ -70,39 +77,34 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         return {
             'bucket': self.bucket,
             'key': self.key,
-            'fileobj': self.filename
+            'fileobj': self.filename,
         }
 
     def create_invalid_extra_args(self):
-        return {
-            'Foo': 'bar'
-        }
+        return {'Foo': 'bar'}
 
     def create_stubbed_responses(self):
         # We want to make sure the beginning of the stream is always used
-        # incase this gets called twice.
+        # in case this gets called twice.
         self.stream.seek(0)
         return [
             {
                 'method': 'head_object',
                 'service_response': {
-                    'ContentLength': len(self.content)
-                }
+                    'ContentLength': len(self.content),
+                    'ETag': self.etag,
+                },
             },
             {
                 'method': 'get_object',
-                'service_response': {
-                    'Body': self.stream
-                }
-            }
+                'service_response': {'Body': self.stream},
+            },
         ]
 
     def create_expected_progress_callback_info(self):
         # Note that last read is from the empty sentinel indicating
         # that the stream is done.
-        return [
-            {'bytes_transferred': 10}
-        ]
+        return [{'bytes_transferred': 10}]
 
     def add_head_object_response(self, expected_params=None):
         head_response = self.create_stubbed_responses()[0]
@@ -111,17 +113,23 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         self.stubber.add_response(**head_response)
 
     def add_successful_get_object_responses(
-            self, expected_params=None, expected_ranges=None):
+        self, expected_params=None, expected_ranges=None, extras=None
+    ):
         # Add all get_object responses needed to complete the download.
         # Should account for both ranged and nonranged downloads.
         for i, stubbed_response in enumerate(
-                self.create_stubbed_responses()[1:]):
+            self.create_stubbed_responses()[1:]
+        ):
             if expected_params:
                 stubbed_response['expected_params'] = copy.deepcopy(
-                    expected_params)
+                    expected_params
+                )
                 if expected_ranges:
-                    stubbed_response['expected_params'][
-                        'Range'] = expected_ranges[i]
+                    stubbed_response['expected_params']['Range'] = (
+                        expected_ranges[i]
+                    )
+                if extras:
+                    stubbed_response['service_response'].update(extras[i])
             self.stubber.add_response(**stubbed_response)
 
     def add_n_retryable_get_object_responses(self, n, num_reads=0):
@@ -130,8 +138,9 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
                 method='get_object',
                 service_response={
                     'Body': StreamWithError(
-                        copy.deepcopy(self.stream), SOCKET_ERROR, num_reads)
-                }
+                        copy.deepcopy(self.stream), SOCKET_ERROR, num_reads
+                    )
+                },
             )
 
     def test_download_temporary_file_does_not_exist(self):
@@ -143,7 +152,7 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         # Make sure the file exists
         self.assertTrue(os.path.exists(self.filename))
         # Make sure the random temporary file does not exist
-        possible_matches = glob.glob('%s*' % self.filename + os.extsep)
+        possible_matches = glob.glob(f'{self.filename}*' + os.extsep)
         self.assertEqual(possible_matches, [])
 
     def test_download_for_fileobj(self):
@@ -152,7 +161,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
 
         with open(self.filename, 'wb') as f:
             future = self.manager.download(
-                self.bucket, self.key, f, self.extra_args)
+                self.bucket, self.key, f, self.extra_args
+            )
             future.result()
 
         # Ensure that the contents are correct
@@ -165,10 +175,11 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
 
         # Create a file-like object to test. In this case, it is a BytesIO
         # object.
-        bytes_io = six.BytesIO()
+        bytes_io = BytesIO()
 
         future = self.manager.download(
-            self.bucket, self.key, bytes_io, self.extra_args)
+            self.bucket, self.key, bytes_io, self.extra_args
+        )
         future.result()
 
         # Ensure that the contents are correct
@@ -181,7 +192,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
 
         with open(self.filename, 'wb') as f:
             future = self.manager.download(
-                self.bucket, self.key, NonSeekableWriter(f), self.extra_args)
+                self.bucket, self.key, NonSeekableWriter(f), self.extra_args
+            )
             future.result()
 
         # Ensure that the contents are correct
@@ -200,7 +212,7 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
             future.result()
         # Make sure the actual file and the temporary do not exist
         # by globbing for the file and any of its extensions
-        possible_matches = glob.glob('%s*' % self.filename)
+        possible_matches = glob.glob(f'{self.filename}*')
         self.assertEqual(possible_matches, [])
 
     def test_download_with_nonexistent_directory(self):
@@ -209,7 +221,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
 
         call_kwargs = self.create_call_kwargs()
         call_kwargs['fileobj'] = os.path.join(
-            self.tempdir, 'missing-directory', 'myfile')
+            self.tempdir, 'missing-directory', 'myfile'
+        )
         future = self.manager.download(**call_kwargs)
         with self.assertRaises(IOError):
             future.result()
@@ -263,7 +276,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         # currently provide to it to simulate rewinds of callbacks.
         self.config.io_chunksize = 3
         future = self.manager.download(
-            subscribers=[recorder_subscriber], **self.create_call_kwargs())
+            subscribers=[recorder_subscriber], **self.create_call_kwargs()
+        )
         future.result()
 
         # Ensure that there is no more remaining responses and that contents
@@ -275,22 +289,26 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         # Assert that the number of bytes seen is equal to the length of
         # downloaded content.
         self.assertEqual(
-            recorder_subscriber.calculate_bytes_seen(), len(self.content))
+            recorder_subscriber.calculate_bytes_seen(), len(self.content)
+        )
 
         # Also ensure that the second progress invocation was negative three
-        # becasue a retry happened on the second read of the stream and we
+        # because a retry happened on the second read of the stream and we
         # know that the chunk size for each read is 3.
         progress_byte_amts = [
-            call['bytes_transferred'] for call in
-            recorder_subscriber.on_progress_calls
+            call['bytes_transferred']
+            for call in recorder_subscriber.on_progress_calls
         ]
         self.assertEqual(-3, progress_byte_amts[1])
 
-    def test_can_provide_file_size(self):
+    def test_can_provide_file_size_and_etag(self):
         self.add_successful_get_object_responses()
 
         call_kwargs = self.create_call_kwargs()
-        call_kwargs['subscribers'] = [FileSizeProvider(len(self.content))]
+        call_kwargs['subscribers'] = [
+            FileSizeProvider(len(self.content)),
+            ETagProvider(self.etag),
+        ]
 
         future = self.manager.download(**call_kwargs)
         future.result()
@@ -319,7 +337,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
 
     @skip_if_windows('Windows does not support UNIX special files')
     @skip_if_using_serial_implementation(
-        'A seperate thread is needed to read from the fifo')
+        'A separate thread is needed to read from the fifo'
+    )
     def test_download_for_fifo_file(self):
         self.add_head_object_response()
         self.add_successful_get_object_responses()
@@ -328,7 +347,8 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
         os.mkfifo(self.filename)
 
         future = self.manager.download(
-            self.bucket, self.key, self.filename, self.extra_args)
+            self.bucket, self.key, self.filename, self.extra_args
+        )
 
         # The call to open a fifo will block until there is both a reader
         # and a writer, so we need to open it for reading after we've
@@ -337,6 +357,16 @@ class BaseDownloadTest(BaseGeneralInterfaceTest):
             future.result()
             self.assertEqual(fifo.read(), self.content)
 
+    def test_raise_exception_on_s3_object_lambda_resource(self):
+        s3_object_lambda_arn = (
+            'arn:aws:s3-object-lambda:us-west-2:123456789012:'
+            'accesspoint:my-accesspoint'
+        )
+        with self.assertRaisesRegex(ValueError, 'methods do not support'):
+            self.manager.download(
+                s3_object_lambda_arn, self.key, self.filename, self.extra_args
+            )
+
 
 class TestNonRangedDownload(BaseDownloadTest):
     # TODO: If you want to add tests outside of this test class and still
@@ -351,12 +381,31 @@ class TestNonRangedDownload(BaseDownloadTest):
         expected_params = {
             'Bucket': self.bucket,
             'Key': self.key,
-            'RequestPayer': 'requester'
+            'RequestPayer': 'requester',
         }
         self.add_head_object_response(expected_params)
         self.add_successful_get_object_responses(expected_params)
         future = self.manager.download(
-            self.bucket, self.key, self.filename, self.extra_args)
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        future.result()
+
+        # Ensure that the contents are correct
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.content, f.read())
+
+    def test_download_with_checksum_enabled(self):
+        self.extra_args['ChecksumMode'] = 'ENABLED'
+        expected_params = {
+            'Bucket': self.bucket,
+            'Key': self.key,
+            'ChecksumMode': 'ENABLED',
+        }
+        self.add_head_object_response(expected_params)
+        self.add_successful_get_object_responses(expected_params)
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
         future.result()
 
         # Ensure that the contents are correct
@@ -370,17 +419,48 @@ class TestNonRangedDownload(BaseDownloadTest):
 
     def test_download_empty_object(self):
         self.content = b''
-        self.stream = six.BytesIO(self.content)
+        self.stream = BytesIO(self.content)
         self.add_head_object_response()
         self.add_successful_get_object_responses()
         future = self.manager.download(
-            self.bucket, self.key, self.filename, self.extra_args)
+            self.bucket, self.key, self.filename, self.extra_args
+        )
         future.result()
 
         # Ensure that the empty file exists
         with open(self.filename, 'rb') as f:
             self.assertEqual(b'', f.read())
 
+    def test_uses_bandwidth_limiter(self):
+        self.content = b'a' * 1024 * 1024
+        self.stream = BytesIO(self.content)
+        self.config = TransferConfig(
+            max_request_concurrency=1, max_bandwidth=len(self.content) / 2
+        )
+        self._manager = TransferManager(self.client, self.config)
+
+        self.add_head_object_response()
+        self.add_successful_get_object_responses()
+
+        start = time.time()
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        future.result()
+        # This is just a smoke test to make sure that the limiter is
+        # being used and not necessary its exactness. So we set the maximum
+        # bandwidth to len(content)/2 per sec and make sure that it is
+        # noticeably slower. Ideally it will take more than two seconds, but
+        # given tracking at the beginning of transfers are not entirely
+        # accurate setting at the initial start of a transfer, we give us
+        # some flexibility by setting the expected time to half of the
+        # theoretical time to take.
+        self.assertGreaterEqual(time.time() - start, 1)
+
+        # Ensure that the contents are correct
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.content, f.read())
+
 
 class TestRangedDownload(BaseDownloadTest):
     # TODO: If you want to add tests outside of this test class and still
@@ -391,10 +471,12 @@ class TestRangedDownload(BaseDownloadTest):
     __test__ = True
 
     def setUp(self):
-        super(TestRangedDownload, self).setUp()
+        super().setUp()
         self.config = TransferConfig(
-            max_request_concurrency=1, multipart_threshold=1,
-            multipart_chunksize=4)
+            max_request_concurrency=1,
+            multipart_threshold=1,
+            multipart_chunksize=4,
+        )
         self._manager = TransferManager(self.client, self.config)
 
     def create_stubbed_responses(self):
@@ -402,27 +484,22 @@ class TestRangedDownload(BaseDownloadTest):
             {
                 'method': 'head_object',
                 'service_response': {
-                    'ContentLength': len(self.content)
-                }
+                    'ContentLength': len(self.content),
+                    'ETag': self.etag,
+                },
             },
             {
                 'method': 'get_object',
-                'service_response': {
-                    'Body': six.BytesIO(self.content[0:4])
-                }
+                'service_response': {'Body': BytesIO(self.content[0:4])},
             },
             {
                 'method': 'get_object',
-                'service_response': {
-                    'Body': six.BytesIO(self.content[4:8])
-                }
+                'service_response': {'Body': BytesIO(self.content[4:8])},
             },
             {
                 'method': 'get_object',
-                'service_response': {
-                    'Body': six.BytesIO(self.content[8:])
-                }
-            }
+                'service_response': {'Body': BytesIO(self.content[8:])},
+            },
         ]
 
     def create_expected_progress_callback_info(self):
@@ -437,15 +514,134 @@ class TestRangedDownload(BaseDownloadTest):
         expected_params = {
             'Bucket': self.bucket,
             'Key': self.key,
-            'RequestPayer': 'requester'
+            'RequestPayer': 'requester',
         }
         expected_ranges = ['bytes=0-3', 'bytes=4-7', 'bytes=8-']
+        stubbed_ranges = ['bytes 0-3/10', 'bytes 4-7/10', 'bytes 8-9/10']
         self.add_head_object_response(expected_params)
         self.add_successful_get_object_responses(
-            expected_params, expected_ranges)
+            {**expected_params, 'IfMatch': self.etag},
+            expected_ranges,
+            [{"ContentRange": r} for r in stubbed_ranges],
+        )
 
         future = self.manager.download(
-            self.bucket, self.key, self.filename, self.extra_args)
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        future.result()
+
+        # Ensure that the contents are correct
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.content, f.read())
+
+    def test_download_with_checksum_enabled(self):
+        self.extra_args['ChecksumMode'] = 'ENABLED'
+        expected_params = {
+            'Bucket': self.bucket,
+            'Key': self.key,
+            'ChecksumMode': 'ENABLED',
+        }
+        expected_ranges = ['bytes=0-3', 'bytes=4-7', 'bytes=8-']
+        self.add_head_object_response(expected_params)
+        self.add_successful_get_object_responses(
+            {**expected_params, 'IfMatch': self.etag}, expected_ranges
+        )
+
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        future.result()
+
+        # Ensure that the contents are correct
+        with open(self.filename, 'rb') as f:
+            self.assertEqual(self.content, f.read())
+
+    def test_download_raises_if_content_range_mismatch(self):
+        expected_params = {
+            'Bucket': self.bucket,
+            'Key': self.key,
+        }
+        expected_ranges = ['bytes=0-3', 'bytes=4-7', 'bytes=8-']
+        # Note that the final retrieved range should be `bytes 8-9/10`.
+        stubbed_ranges = ['bytes 0-3/10', 'bytes 4-7/10', 'bytes 7-8/10']
+        self.add_head_object_response(expected_params)
+        self.add_successful_get_object_responses(
+            {**expected_params, 'IfMatch': self.etag},
+            expected_ranges,
+            [{"ContentRange": r} for r in stubbed_ranges],
+        )
+
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        with self.assertRaises(S3ValidationError) as e:
+            future.result()
+        self.assertIn('does not match content range', str(e.exception))
+
+    def test_download_raises_if_etag_validation_fails(self):
+        expected_params = {
+            'Bucket': self.bucket,
+            'Key': self.key,
+        }
+        expected_ranges = ['bytes=0-3', 'bytes=4-7']
+        self.add_head_object_response(expected_params)
+
+        # Add successful GetObject responses for the first 2 requests.
+        for i, stubbed_response in enumerate(
+            self.create_stubbed_responses()[1:3]
+        ):
+            stubbed_response['expected_params'] = copy.deepcopy(
+                {**expected_params, 'IfMatch': self.etag}
+            )
+            stubbed_response['expected_params']['Range'] = expected_ranges[i]
+            self.stubber.add_response(**stubbed_response)
+
+        # Simulate ETag validation failure by adding a
+        # client error for the last GetObject request.
+        self.stubber.add_client_error(
+            method='get_object',
+            service_error_code='PreconditionFailed',
+            service_message=(
+                'At least one of the pre-conditions you specified did not hold'
+            ),
+            http_status_code=412,
+        )
+
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
+        with self.assertRaises(S3DownloadFailedError) as e:
+            future.result()
+        self.assertIn('did not match expected ETag', str(e.exception))
+
+        # Ensure no data is written to disk.
+        self.assertFalse(os.path.exists(self.filename))
+
+    def test_download_without_etag(self):
+        expected_params = {
+            'Bucket': self.bucket,
+            'Key': self.key,
+        }
+        expected_ranges = ['bytes=0-3', 'bytes=4-7', 'bytes=8-']
+
+        # Stub HeadObject response with no ETag
+        head_object_response = {
+            'method': 'head_object',
+            'service_response': {
+                'ContentLength': len(self.content),
+            },
+            'expected_params': expected_params,
+        }
+        self.stubber.add_response(**head_object_response)
+
+        # This asserts that IfMatch isn't in the GetObject requests.
+        self.add_successful_get_object_responses(
+            expected_params, expected_ranges
+        )
+
+        future = self.manager.download(
+            self.bucket, self.key, self.filename, self.extra_args
+        )
         future.result()
 
         # Ensure that the contents are correct
