@@ -38,13 +38,13 @@ from s3transfer.utils import (
     SlidingWindowSemaphore,
     StreamReaderProgress,
     TaskSemaphore,
-    add_s3express_defaults,
     calculate_num_parts,
     calculate_range_parameter,
     get_callbacks,
     get_filtered_dict,
     invoke_progress_callbacks,
     random_file_extension,
+    set_default_checksum_algorithm,
 )
 from tests import NonSeekableWriter, RecordingSubscriber, mock, unittest
 
@@ -282,7 +282,7 @@ class TestOSUtils(BaseUtilsTest):
         try:
             OSUtils().remove_file(non_existent_file)
         except OSError as e:
-            self.fail('OSError should have been caught: %s' % e)
+            self.fail(f'OSError should have been caught: {e}')
 
     def test_remove_file_proxies_remove_file(self):
         OSUtils().remove_file(self.filename)
@@ -306,7 +306,7 @@ class TestOSUtils(BaseUtilsTest):
         filename = 'myfile'
         self.assertIsNotNone(
             re.match(
-                r'%s\.[0-9A-Fa-f]{8}$' % filename,
+                rf'{filename}\.[0-9A-Fa-f]{{8}}$',
                 OSUtils().get_temp_filename(filename),
             )
         )
@@ -329,7 +329,7 @@ class TestOSUtils(BaseUtilsTest):
 
     @mock.patch('s3transfer.utils.fallocate')
     def test_allocate_with_io_error(self, mock_fallocate):
-        mock_fallocate.side_effect = IOError()
+        mock_fallocate.side_effect = OSError()
         with self.assertRaises(IOError):
             OSUtils().allocate(self.filename, 1)
         self.assertFalse(os.path.exists(self.filename))
@@ -1192,32 +1192,44 @@ class TestAdjustChunksize(unittest.TestCase):
         self.assertEqual(new_size, MAX_SINGLE_UPLOAD_SIZE)
 
 
-class TestS3ExpressDefaults:
+class TestS3Defaults:
     @pytest.mark.parametrize(
         "bucket,extra_args,expected",
         (
             (
                 "mytestbucket--usw2-az2--x-s3",
                 {},
-                {"ChecksumAlgorithm": "crc32"},
+                {"ChecksumAlgorithm": "CRC32"},
             ),
             (
                 "mytestbucket--usw2-az2--x-s3",
                 {"Some": "Setting"},
-                {"ChecksumAlgorithm": "crc32", "Some": "Setting"},
+                {"ChecksumAlgorithm": "CRC32", "Some": "Setting"},
+            ),
+            (
+                "mytestbucket--usw2-az2--x-s3",
+                {"ChecksumAlgorithm": "sha256"},
+                {"ChecksumAlgorithm": "sha256"},
             ),
             (
                 "mytestbucket",
                 {},
-                {},
+                {"ChecksumAlgorithm": "CRC32"},
             ),
             (
-                "mytestbucket--usw2-az2--x-s3",
+                "mytestbucket",
+                {"Some": "Setting"},
+                {"ChecksumAlgorithm": "CRC32", "Some": "Setting"},
+            ),
+            (
+                "mytestbucket",
                 {"ChecksumAlgorithm": "sha256"},
                 {"ChecksumAlgorithm": "sha256"},
             ),
         ),
     )
-    def test_add_s3express_defaults(self, bucket, extra_args, expected):
-        add_s3express_defaults(bucket, extra_args)
+    def test_set_default_checksum_algorithm(
+        self, bucket, extra_args, expected
+    ):
+        set_default_checksum_algorithm(extra_args)
         assert extra_args == expected
